@@ -16,7 +16,7 @@ from dgraph_orm.vendors.edgedb import (
 )
 
 client = create_async_client(
-    tls_security="insecure", host="143.244.174.160", password="beatgig8859"
+    tls_security="insecure", host="143.244.174.160", password="beatgig8859", port=5656
 )
 
 
