@@ -574,8 +574,9 @@ async def test_extra_fields():
         .limit(100)
         .offset(1)
         .extra_fields(
-            "artist_name_external := [is ExternalBooking].artist_name, "
-            "artist_name := [is Booking].artist.name, artist_slug := [is Booking].artist.slug,"
+            "is_external := exists [is ExternalBooking].id, "
+            "artist_name := [is ExternalBooking].artist_name ?? [is Booking].artist.name, "
+            "artist_slug := [is Booking].artist.slug, "
             "cover_photo_external := [is ExternalBooking].cover_image, "
             "cover_image := [is Booking].artist.cover_image, artist_bio := [is Booking].artist.bio"
         )
@@ -587,12 +588,11 @@ async def test_extra_fields():
         calendar_bookings.append(
             CalendarBooking(
                 firebase_id=meta.firebase_id,
-                artist_name=meta.extra.get("artist_name_external")
-                or meta.extra.get("artist_name"),
+                artist_name=meta.extra.get("artist_name"),
                 start_time=meta.start_time,
-                booking_type=CalendarBookingType.internal
-                if meta.extra.get("artist_name")
-                else CalendarBookingType.external,
+                booking_type=CalendarBookingType.external
+                if meta.extra.get("is_external")
+                else CalendarBookingType.internal,
                 artist_bio=meta.extra.get("artist_bio"),
                 public_event_description=meta.public_event_description,
                 performance_length_mins=meta.performance_length_mins,
@@ -602,6 +602,93 @@ async def test_extra_fields():
     debug(calendar_bookings)
 
 
+async def test_bulk_reads():
+    users = await DB.UserResolver().query()
+    slugs = [u.slug for u in users]
+
+    def create_proms(given_slugs):
+        return [
+            DB.UserResolver()
+            .artists(DB.ArtistResolver().bookings(DB.BookingResolver().limit(10)))
+            .venues(DB.VenueResolver().bookings(DB.BookingResolver().limit(10)))
+            .gerror(slug=slug)
+            for slug in given_slugs
+        ]
+
+    all_users_again = []
+    start = time.time()
+    for x in range(5):
+        users_again = await asyncio.gather(*create_proms(slugs))
+        all_users_again.extend(users_again)
+    took = time.time() - start
+    print(f"bulk reading {took=}")
+    print(len(users_again))
+
+
+async def investigate_bulk_load_bug():
+    from dgraph_orm.vendors.edgedb.gen import local as db
+
+    # db = DB
+
+    batch = Batch(client=db.client)
+    inserts = [
+        db.PersonInsert(first_name="jay", slug=random_str(10)) for _ in range(50)
+    ]
+    print("starting")
+    [await db.Person.add(insert=i, batch=batch) for i in inserts]
+    res = await batch.commit()
+    debug(res)
+
+
+async def investigate_bulk_load_custom_code():
+    import random
+    import string
+
+    def random_str(n: int) -> str:
+        return "".join(random.choices(string.ascii_letters + string.digits, k=n))
+
+    insert_strs: T.List[str] = []
+    model_names: T.List[str] = []
+    n = 50
+    for i in range(n):
+        model_name = f"model_{i}"
+        insert_strs.append(
+            f"{model_name}: (insert Person {{ first_name := <str>$first_name_{i}, slug := <str>$slug_{i} }} )"
+        )
+
+    union_str = f'models := ({" union ".join(model_names)})'
+    insert_str = ",\n".join([*insert_strs, union_str])
+    query_str = f"""
+with
+{insert_str}
+select models {{id}}
+    """
+    first_name_variables = {f"first_name_{i}": random_str(10) for i in range(n)}
+    variables = {}
+
+
+async def using_for():
+    n = 1000
+    query_s = f"""
+with
+    raw_data := <json>$data,
+for item in json_array_unpack(raw_data) union (
+    insert Person {{ first_name := <str>item['first_name'], slug := <str>item['slug'], last_name := <str>item['last_name'] }}
+) {{id, slug, last_name, created_at}}
+        """
+    data = [
+        {
+            "first_name": random_str(10),
+            "slug": random_str(10),
+            "last_name": "Scooby" if i % 2 == 0 else None,
+        }
+        for i in range(n)
+    ]
+    variables = {"data": json.dumps(data)}
+    res = await execute_query(client=DB.client, query_str=query_s, variables=variables)
+    debug(res)
+
+
 async def main():
     # await test_get()
     # await test_nested()
@@ -621,7 +708,10 @@ async def main():
     # await test_generated_add()
     # await test_generated_delete()
 
-    await test_extra_fields()
+    # await test_extra_fields()
+    # await test_bulk_reads()
+    # await investigate_bulk_load_bug()
+    await using_for()
 
     # expensive ones
     # await add_lots_async()
